/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Clippings.
 *
 * The Initial Developer of the Original Code is 
 * Alex Eng <ateng@users.sourceforge.net>.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */


//
// Clippings version 3.0 initialization library
//
// Dependencies:
//  - utils.js
//  - clippak.js


/*
 * Constructor
 */
function aeClippings3(aClippingsSvc, aStringBundle) 
{
  if (! aClippingsSvc) {
    throw this.E_CTOR_MISSING_ARG;
  }

  this._clippingsSvc = aClippingsSvc;
  this._appUtils = new aeClippingsUtils();
  this._strBundle = aStringBundle;
}

aeClippings3.prototype = {
  _clippingsSvc:  null,
  _appUtils:      null,
  _strBundle:     null,
  E_CTOR_MISSING_ARG:  "nsIClippingsService argument to aeClippings3 constructor is missing or undefined",
  E_CLIPPINGSSVC_NOT_INITIALIZED: "nsIClippingsService not initialized"
};


/*
 * Starts the initialization process
 */
aeClippings3.prototype.startInit = function ()
{
  // TO DO:
  // - Initialize common clippings data
  //
  // See bug 17720 for details.

  this._importPackagedClippings();
  this._processEmptyFolders();
};


/*
 * Import clippings from the packaged data source file, if it exists
 */
aeClippings3.prototype._importPackagedClippings = function ()
{
  var packagedClippings = new aePackagedClippings();
  if (packagedClippings.exists()) {
    this._appUtils.log("It appears that a packaged clipping datasource file exists");

    try {
      packagedClippings.import(this._clippingsSvc);
    }
    catch (e if e == packagedClippings.E_FLUSH_FAILED) {
      this._appUtils.alertEx(this._strBundle.getString("appName"),
			     this._strBundle.getString("errorFlushAfterPDSImport"));
    }
    catch (e) {
      this._appUtils.alertEx(this._strBundle.getString("appName"),
			     this._strBundle.getString("errorDSImportFailure"));
      var consoleSvc = Components.classes["@mozilla.org/consoleservice;1"]
	                         .getService(Components.interfaces
					               .nsIConsoleService);
      var msg = String.ae_fmt("Packaged Clipping import error: %s", e);
      consoleSvc.logStringMessage(msg);
    }
  }
};


/*
 * Process empty subfolders recursively and append the empty clipping
 * to each empty folder
 */
aeClippings3.prototype._processEmptyFolders = function ()
{
  try {
    this._clippingsSvc.processEmptyFolders();
  }
  catch (e) {
    throw this.E_CLIPPINGSSVC_NOT_INITIALIZED;
  }
};

