/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Clippings.
 *
 * The Initial Developer of the Original Code is 
 * Alex Eng <ateng@users.sourceforge.net>.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */

//
// Dependencies:
//  - utils.js
//


// Toolbar command IDs
const CMD_NEW = 1;
const CMD_NEW_FROM_CLIPBOARD = 2;
const CMD_OPEN_CLIPPINGS_MGR = 3;

// Global variables
var gAppUtils;
var gClippingsSvc;
var gTimerID;


//
// Floating toolbar window management
//

var gWindoid = {
  titlebarElt: null,

  focus: function (aEvent)
  {
    this.titlebarElt.style.backgroundColor = "activecaption";
  },

  blur:  function (aEvent)
  {
    this.titlebarElt.style.backgroundColor = "inactivecaption";
  }
};


var gWindoidMove = {
  isMoving:     false,
  startCoords:  null,

  start: function (aEvent) 
  {
    if (aEvent.target.id != "windoid-title-bar") {
      return;
    }

    if (this.isMoving) {
      return;
    }

    this.startCoords = { x: aEvent.clientX, y: aEvent.clientY };
    this.isMoving = true;
  },
 
  stop: function (aEvent) 
  {
    if (this.isMoving) {
      this.startCoords = null;
      this.isMoving = false;
    }
  },

  move: function (aEvent) 
  {
    if (! this.isMoving) {
      return;
    }

    var dx = aEvent.clientX - this.startCoords.x;
    var dy = aEvent.clientY - this.startCoords.y;

    window.moveBy(dx, dy);
  }
};


var gWindoidCloseBtn = {
  mouseDown: function (aEvent) 
  {
    if (aEvent.which != 1) {
      return;
    }
  },

  mouseUp: function (aEvent) 
  {
    if (aEvent.which == 1) {
      setTimeout("window.close()", 80);
    }
  }
};



//
// DOM convenience function
//

function $(aID)
{
  return document.getElementById(aID);
}


//
// Clippings toolbar functions
//

function load()
{
  gAppUtils = new aeClippingsUtils();

  // Get Clippings service object from host app window
  var recentWnd = getRecentHostAppWindow();
  if (! recentWnd) {
    alert("Clippings toolbar error.");
    window.close();
    return;
  }

  gClippingsSvc = recentWnd.gClippingsExec.clippingsSvc;
  var ds = gClippingsSvc.getDataSource("");

  // Initialize "Insert Clippings" menu
  var clippingsMenu = $('insert-clippings');
  clippingsMenu.database.AddDataSource(ds);
  clippingsMenu.builder.rebuild();

  gWindoid.titlebarElt = $("windoid-title-bar");
  window.addEventListener("focus", function (e) { gWindoid.focus(e) }, false);
  window.addEventListener("blur", function (e) { gWindoid.blur(e) }, false);

  gTimerID = window.setInterval("checkHostAppWndStatus()", 500);
}


function checkHostAppWndStatus()
{
  var ww = Components.classes["@mozilla.org/embedcomp/window-watcher;1"]
                     .getService(Components.interfaces.nsIWindowWatcher);

  var wnd = ww.activeWindow;
  if (wnd instanceof ChromeWindow) {
    var url = ('location' in wnd) ? wnd.location.href : "(n/a)";
  }  

  if (url != "chrome://browser/content/browser.xul"
      && url != "chrome://messenger/content/messengercompose/messengercompose.xul"
      && url != "chrome://clippings/content/hostappToolbar.xul") {
    var recentWnd = getRecentHostAppWindow();
    recentWnd.gClippingsExec._clippingsToolbar.setHidden(true);
    window.close();
  }
}


function getRecentHostAppWindow()
{
  var rv;
  var recentWnd = gAppUtils.getRecentHostAppWindow();
  rv = ("gClippingsExec" in recentWnd) ? recentWnd : null;

  return rv;
}


function insertClipping(aClippingURI)
{
  var haWnd = getRecentHostAppWindow();

  if (! haWnd) {
    gAppUtils.beep();
    return;
  }

  var name = gClippingsSvc.getName(aClippingURI);
  var text = gClippingsSvc.getText(aClippingURI);
  haWnd.gClippingsExec.insertClippingText(aClippingURI, name, text);
}


function execToolbarCommand(aCommandID)
{
  var haWnd = getRecentHostAppWindow();
  
  if (haWnd) {
    switch (aCommandID) {

    case CMD_NEW_FROM_CLIPBOARD:
      haWnd.gClippingsExec.newFromClipboard();
      return;

    case CMD_OPEN_CLIPPINGS_MGR:
      haWnd.gClippingsExec.openClippingsManager();
      return;
      
    case CMD_NEW:
      haWnd.gClippingsExec.extNewClipping();
      return;
      
    default:
      break;
    }
  }

  gAppUtils.beep();
}


function updateCmds()
{
  // TO DO:
  // If the most recent window isn't a host app window where a clipping can be
  // pasted into, then disable all toolbar buttons except the Organize command.
  // Otherwise, enable them.
}


function unload()
{

}
